@@ -716,7 +716,7 @@ class Response(object):
             status = self.status
 
         if kwheaders:
-            toremove = set(make_header_name(k) for k in kwheaders)
+            toremove = {make_header_name(k) for k in kwheaders}
             headers = [
                 (k, v) for k, v in headers if k not in toremove
             ] + make_headers([], kwheaders)
@@ -787,6 +787,51 @@ class Response(object):
             ],
         )
 
+    @classmethod
+    def error(cls, message="500 Internal Server Error"):
+        """
+        Return an HTTP server error response (500).
+
+        Synopsis::
+
+            >>> def view():
+            ...     return Response.error()
+            ...
+        """
+        return cls(
+            status=STATUS_INTERNAL_SERVER_ERROR,
+            content=[
+                "<html>\n"
+                "<body>\n"
+                f"    <h1>{message}</h1>\n"
+                "</body>\n"
+                "</html>\n"
+            ],
+        )
+
+    @classmethod
+    def unauthorized(cls, authenticate):
+        """
+        Return an HTTP unauthorized response (401)
+        """
+        return cls(
+            "<html>\n"
+            "<body>\n"
+            "<h1>401 Authorization Required</h1>\n"
+            "</body>\n"
+            "</html>",
+            status=STATUS_UNAUTHORIZED,
+            www_authenticate=authenticate
+        )
+
+    @classmethod
+    def unauthorized_basic(cls, realm):
+        """
+        Return an HTTP unauthorized response (401) with a WWW-Authenticate
+        header set for HTTP Basic authentication..
+        """
+        return cls.unauthorized(authenticate=f'Basic realm="{realm}"')
+
     @classmethod
     def forbidden(cls, message="Sorry, access is denied"):
         """\
