@@ -21,6 +21,8 @@ import os
 import pathlib
 import sys
 
+import pytest
+
 from fresco.options import Options
 from fresco.options import parse_key_value_pairs
 
@@ -36,6 +38,16 @@ class TestOptions(object):
         options.x = 1
         assert options.x == 1
 
+    def test_options_raises_AttributeError(self):
+        with pytest.raises(AttributeError):
+            Options().x
+
+    def test_options_works_with_getattr(self):
+        assert getattr(Options(), "x", 1) == 1
+        assert getattr(Options({"x": 2}), "x", 1) == 2
+        with pytest.raises(AttributeError):
+            assert getattr(Options(), "x")
+
     def test_options_update_from_object(self):
         class Foo:
             a = 1
@@ -132,9 +144,7 @@ class TestLoadKeyValuePairs:
 
 
 class TestLoadOptions:
-    def check_loadoptions(
-        self, tmpdir, files, sources="*", tags=[], expected={}
-    ):
+    def check_loadoptions(self, tmpdir, files, sources="*", tags=[], expected={}):
         t = pathlib.Path(tmpdir)
         for fname in files:
             with (t / fname).open("w", encoding="UTF-8") as f:
@@ -143,9 +153,7 @@ class TestLoadOptions:
         @contextlib.contextmanager
         def optionsdir():
             def loadopts(sources="*", tags=[], strict=False, **kw):
-                return Options().load(
-                    str(sources), tags, strict=False, **kw
-                )
+                return Options().load(str(sources), tags, strict=False, **kw)
 
             saved = os.getcwd()
             os.chdir(t)
@@ -165,9 +173,7 @@ class TestLoadOptions:
         self.check_loadoptions(
             tmpdir, {"a": "x = 2\ny = ${x}"}, expected={"x": 2, "y": 2}
         )
-        with self.check_loadoptions(
-            tmpdir, {"a": "x = $__FILE__\ny=${x}"}
-        ) as loadopts:
+        with self.check_loadoptions(tmpdir, {"a": "x = $__FILE__\ny=${x}"}) as loadopts:
             result = loadopts()
             assert result["x"] == result["y"] == str(pathlib.Path(tmpdir) / "a")
 
@@ -202,7 +208,8 @@ class TestLoadOptions:
 
     def test_it_loads_in_tag_order(self, tmpdir):
         with self.check_loadoptions(
-            tmpdir, {"a.dev.txt": "a = dev", "a.local.txt": "a = local"},
+            tmpdir,
+            {"a.dev.txt": "a = dev", "a.local.txt": "a = local"},
         ) as loadopts:
             assert loadopts("*", ["dev", "local"]) == {"a": "local"}
             assert loadopts("*", ["local", "dev"]) == {"a": "dev"}
@@ -226,4 +233,4 @@ class TestLoadOptions:
             with TemporaryDirectory() as tmpdir2:
                 os.chdir(tmpdir2)
                 assert loadopts("*") == {}
-                assert loadopts("*", dir=tmpdir) == {'a': 1}
+                assert loadopts("*", dir=tmpdir) == {"a": 1}
