@@ -13,7 +13,7 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-from mock import Mock
+from unittest.mock import Mock
 import pytest
 
 from fresco import FrescoApp
@@ -93,6 +93,12 @@ class TestCannedResponses(object):
         assert b"not found" in b"".join(r.content_iterator).lower()
         assert r.status_code == 404
 
+    def test_error(self):
+        with FrescoApp().requestcontext():
+            r = Response.error()
+        assert b"internal server error" in b"".join(r.content_iterator).lower()
+        assert r.status_code == 500
+
     def test_method_not_allowed(self):
         with FrescoApp().requestcontext():
             r = Response.method_not_allowed(valid_methods=("PUT", "DELETE"))
@@ -100,6 +106,18 @@ class TestCannedResponses(object):
         assert r.status == "405 Method Not Allowed"
         assert r.get_header("Allow") == "PUT,DELETE"
 
+    def test_unauthorized(self):
+        with FrescoApp().requestcontext():
+            r = Response.unauthorized(authenticate='myscheme realm="spaghetti"')
+        assert r.status == "401 Unauthorized"
+        assert r.get_header("WWW-Authenticate") == 'myscheme realm="spaghetti"'
+
+    def test_unauthorized_basic(self):
+        with FrescoApp().requestcontext():
+            r = Response.unauthorized_basic(realm="spaghetti")
+        assert r.status == "401 Unauthorized"
+        assert r.get_header("WWW-Authenticate") == 'Basic realm="spaghetti"'
+
     def test_meta_refresh(self):
         with FrescoApp().requestcontext():
             r = Response.meta_refresh("/next_page")
