@@ -17,7 +17,7 @@ import os
 import re
 from calendar import timegm
 from email.utils import formatdate, parsedate
-from mock import Mock
+from unittest.mock import Mock
 from tempfile import mkstemp
 from fresco import FrescoApp
 from fresco.static import serve_static_file
