@@ -56,14 +56,14 @@ class TestNormPath(object):
 
 class TestMakeQuery(object):
     def test_make_query(self):
-        assert sorted(make_query(a="1", b=2).split(";")) == ["a=1", "b=2"]
+        assert sorted(make_query(a="1", b=2).split("&")) == ["a=1", "b=2"]
         assert make_query(a="one two three") == "a=one+two+three"
-        assert make_query(a=["one", "two", "three"]) == "a=one;a=two;a=three"
+        assert make_query(a=["one", "two", "three"]) == "a=one&a=two&a=three"
 
     def test_make_query_unicode(self):
         assert (
             make_query(a=[alpha, beta, gamma], charset="utf8")
-            == "a=%CE%B1;a=%CE%B2;a=%CE%B3"
+            == "a=%CE%B1&a=%CE%B2&a=%CE%B3"
         )
 
     def test_make_query_unicode_default_encoding(self):
