@@ -19,357 +19,41 @@ Utilities for working with data on the HTTP level
 
 from binascii import hexlify
 from collections import namedtuple
+from collections import deque
 from email.header import Header
 from email.message import Message
-from email.parser import BytesParser
-from functools import partial
+from email.parser import BytesFeedParser
 from io import BytesIO
 from itertools import chain
 from typing import Dict
 from typing import Iterator
+from typing import Iterable
+from typing import List
 from typing import Tuple
+from typing import Callable
 from typing import Union
+from tempfile import SpooledTemporaryFile
 import os
 import re
+import io
 
 from urllib.parse import unquote_plus
 from shutil import copyfileobj
 
 import fresco
 from fresco.exceptions import RequestParseError
-from fresco.util.io import (
-    ExpandableOutput,
-    SizeLimitedInput,
-    PutbackInput,
-    DelimitedInput,
-)
+from fresco.util.io import io_iterator
+from fresco.util.io import ByteIterator
 from fresco.util.wsgi import str_to_environ
+from fresco.util.contentencodings import ALLOWED_ENCODINGS
 
 KB = 1024
 MB = 1024 * KB
 
 #: Data chunk size to read from the input stream (wsgi.input)
-CHUNK_SIZE = 8 * KB
+CHUNK_SIZE = min(io.DEFAULT_BUFFER_SIZE, 1024)
 
-#: Allowed character encodings.
-#: This list is based on the list of standard encodings here:
-#:
-#:     https://docs.python.org/3.4/library/codecs.html
-#:
-#: Python specific encodings and binary transformations (eg zlib decompression)
-#: are excluded to avoid zip bomb style attacks.
-ALLOWED_ENCODINGS = set(
-    [
-        "ascii",
-        "646",
-        "us-ascii",
-        "big5",
-        "big5-tw",
-        "csbig5",
-        "big5hkscs",
-        "big5-hkscs",
-        "hkscs",
-        "cp037",
-        "IBM037",
-        "IBM039",
-        "cp273",
-        "273",
-        "IBM273",
-        "csIBM273",
-        "cp424",
-        "EBCDIC-CP-HE",
-        "IBM424",
-        "cp437",
-        "437",
-        "IBM437",
-        "cp500",
-        "EBCDIC-CP-BE",
-        "EBCDIC-CP-CH",
-        "IBM500",
-        "cp720",
-        "Arabic",
-        "cp737",
-        "Greek",
-        "cp775",
-        "IBM775",
-        "cp850",
-        "850",
-        "IBM850",
-        "cp852",
-        "852",
-        "IBM852",
-        "cp855",
-        "855",
-        "IBM855",
-        "cp856",
-        "Hebrew",
-        "cp857",
-        "857",
-        "IBM857",
-        "cp858",
-        "858",
-        "IBM858",
-        "cp860",
-        "860",
-        "IBM860",
-        "cp861",
-        "861",
-        "CP-IS",
-        "IBM861",
-        "cp862",
-        "862",
-        "IBM862",
-        "cp863",
-        "863",
-        "IBM863",
-        "cp864",
-        "IBM864",
-        "cp865",
-        "865",
-        "IBM865",
-        "cp866",
-        "866",
-        "IBM866",
-        "cp869",
-        "869",
-        "CP-GR",
-        "IBM869",
-        "cp874",
-        "Thai",
-        "cp875",
-        "Greek",
-        "cp932",
-        "932",
-        "ms932",
-        "mskanji",
-        "ms-kanji",
-        "cp949",
-        "949",
-        "ms949",
-        "uhc",
-        "cp950",
-        "950",
-        "ms950",
-        "cp1006",
-        "Urdu",
-        "cp1026",
-        "ibm1026",
-        "cp1125",
-        "1125",
-        "ibm1125",
-        "cp866u",
-        "ruscii",
-        "cp1140",
-        "ibm1140",
-        "cp1250",
-        "windows-1250",
-        "cp1251",
-        "windows-1251",
-        "cp1252",
-        "windows-1252",
-        "cp1253",
-        "windows-1253",
-        "cp1254",
-        "windows-1254",
-        "cp1255",
-        "windows-1255",
-        "cp1256",
-        "windows-1256",
-        "cp1257",
-        "windows-1257",
-        "cp1258",
-        "windows-1258",
-        "cp65001",
-        "euc_jp",
-        "eucjp",
-        "ujis",
-        "u-jis",
-        "euc_jis_2004",
-        "jisx0213",
-        "eucjis2004",
-        "euc_jisx0213",
-        "eucjisx0213",
-        "euc_kr",
-        "euckr",
-        "korean",
-        "ksc5601",
-        "ks_c-5601",
-        "ks_c-5601-1987",
-        "ksx1001",
-        "ks_x-1001",
-        "gb2312",
-        "chinese",
-        "csiso58gb231280",
-        "euc-" "cn",
-        "euccn",
-        "eucgb2312-cn",
-        "gb2312-1980",
-        "gb2312-80",
-        "iso-" "ir-58",
-        "gbk",
-        "936",
-        "cp936",
-        "ms936",
-        "gb18030",
-        "gb18030-2000",
-        "hz",
-        "hzgb",
-        "hz-gb",
-        "hz-gb-2312",
-        "iso2022_jp",
-        "csiso2022jp",
-        "iso2022jp",
-        "iso-2022-jp",
-        "iso2022_jp_1",
-        "iso2022jp-1",
-        "iso-2022-jp-1",
-        "iso2022_jp_2",
-        "iso2022jp-2",
-        "iso-2022-jp-2",
-        "iso2022_jp_2004",
-        "iso2022jp-2004",
-        "iso-2022-jp-2004",
-        "iso2022_jp_3",
-        "iso2022jp-3",
-        "iso-2022-jp-3",
-        "iso2022_jp_ext",
-        "iso2022jp-ext",
-        "iso-2022-jp-ext",
-        "iso2022_kr",
-        "csiso2022kr",
-        "iso2022kr",
-        "iso-2022-kr",
-        "latin_1",
-        "iso-8859-1",
-        "iso8859-1",
-        "8859",
-        "cp819",
-        "latin",
-        "latin1",
-        "L1",
-        "iso8859_2",
-        "iso-8859-2",
-        "latin2",
-        "L2",
-        "iso8859_3",
-        "iso-8859-3",
-        "latin3",
-        "L3",
-        "iso8859_4",
-        "iso-8859-4",
-        "latin4",
-        "L4",
-        "iso8859_5",
-        "iso-8859-5",
-        "cyrillic",
-        "iso8859_6",
-        "iso-8859-6",
-        "arabic",
-        "iso8859_7",
-        "iso-8859-7",
-        "greek",
-        "greek8",
-        "iso8859_8",
-        "iso-8859-8",
-        "hebrew",
-        "iso8859_9",
-        "iso-8859-9",
-        "latin5",
-        "L5",
-        "iso8859_10",
-        "iso-8859-10",
-        "latin6",
-        "L6",
-        "iso8859_13",
-        "iso-8859-13",
-        "latin7",
-        "L7",
-        "iso8859_14",
-        "iso-8859-14",
-        "latin8",
-        "L8",
-        "iso8859_15",
-        "iso-8859-15",
-        "latin9",
-        "L9",
-        "iso8859_16",
-        "iso-8859-16",
-        "latin10",
-        "L10",
-        "johab",
-        "cp1361",
-        "ms1361",
-        "koi8_r",
-        "Russian",
-        "koi8_u",
-        "Ukrainian",
-        "mac_cyrillic",
-        "maccyrillic",
-        "mac_greek",
-        "macgreek",
-        "mac_iceland",
-        "maciceland",
-        "mac_latin2",
-        "maclatin2",
-        "maccentraleurope",
-        "mac_roman",
-        "macroman",
-        "macintosh",
-        "mac_turkish",
-        "macturkish",
-        "ptcp154",
-        "csptcp154",
-        "pt154",
-        "cp154",
-        "cyrillic-asian",
-        "shift_jis",
-        "csshiftjis",
-        "shiftjis",
-        "sjis",
-        "s_jis",
-        "shift_jis_2004",
-        "shiftjis2004",
-        "sjis_2004",
-        "sjis2004",
-        "shift_jisx0213",
-        "shiftjisx0213",
-        "sjisx0213",
-        "s_jisx0213",
-        "utf_32",
-        "U32",
-        "utf32",
-        "utf_32_be",
-        "UTF-32BE",
-        "utf_32_le",
-        "UTF-32LE",
-        "utf_16",
-        "U16",
-        "utf16",
-        "utf_16_be",
-        "UTF-16BE",
-        "utf_16_le",
-        "UTF-16LE",
-        "utf_7",
-        "U7",
-        "unicode-1-1-utf-7",
-        "utf_8",
-        "U8",
-        "UTF",
-        "utf8",
-        "utf_8_sig",
-    ]
-)
-
-# "Notice that spelling alternatives that only differ in case or use a hyphen
-# instead of an underscore are also valid aliases; therefore, e.g. 'utf-8' is a
-# valid alias for the 'utf_8' codec."
-ALLOWED_ENCODINGS = set(
-    item.lower().replace("_", "-") for item in ALLOWED_ENCODINGS
-)
-
-ParsedContentType = namedtuple(
-    "ParsedContentType", "content_type encoding params"
-)
+ParsedContentType = namedtuple("ParsedContentType", "content_type encoding params")
 
 token_pattern = r"[!#-\'*-.0-9A-Z\^-~]+"
 quotedstringparts_pattern = r'(?:(\\.)|([^"\\]+))'
@@ -505,9 +189,7 @@ def parse_header(
     value, remaining = header.split(";", 1)
     return (
         value,
-        parse_parameters(
-            remaining.strip(), preserve_backslashes=preserve_backslashes
-        ),
+        parse_parameters(remaining.strip(), preserve_backslashes=preserve_backslashes),
     )
 
 
@@ -516,13 +198,12 @@ def parse_querystring(
     charset: str = None,
     strict: bool = False,
     keep_blank_values: bool = True,
-    pairsplitter=re.compile("[;&]").split,
-) -> Iterator[Tuple[str, str]]:
-    """\
+) -> List[Tuple[str, str]]:
+    """
     Return ``(key, value)`` pairs from the given querystring::
 
         >>> list(parse_querystring('green%20eggs=ham;me=sam+i+am'))
-        [(u'green eggs', u'ham'), (u'me', u'sam i am')]
+        [('green eggs', 'ham'), ('me', 'sam i am')]
 
     :param data: The query string to parse.
     :param charset: Character encoding used to decode values. If not specified,
@@ -532,33 +213,34 @@ def parse_querystring(
                               returned as empty strings. if False, no key,
                               value pair will be returned.
 
-    :param strict: if ``True``, a ``ValueError`` will be raised on parsing
-                   errors.
+    :param strict: if ``True``, a ``RequestParseError`` will be raised on
+                   parsing errors.
     """
 
     if charset is None:
         charset = fresco.DEFAULT_CHARSET
 
-    unquote = partial(unquote_plus, encoding=charset)
+    result: List[Tuple[str, str]] = []
+    append = result.append
+    sep = "&" if "&" in data else ";"
 
-    for item in pairsplitter(data):
+    for item in data.split(sep):
         if not item:
             continue
         try:
             key, value = item.split("=", 1)
         except ValueError:
             if strict:
-                raise RequestParseError("bad query field: %r" % (item,))
+                raise RequestParseError(f"Bad query field: {item}")
             if not keep_blank_values:
                 continue
             key, value = item, ""
 
         try:
-            yield (unquote(key), unquote(value))
+            append((unquote_plus(key, charset), unquote_plus(value, charset)))
         except UnicodeDecodeError:
-            raise RequestParseError(
-                "Invalid character data: can't decode" " as %r" % (charset,)
-            )
+            raise RequestParseError(f"Invalid {charset} character data")
+    return result
 
 
 def parse_post(
@@ -568,7 +250,7 @@ def parse_post(
     max_size=16 * KB,
     max_multipart_size=2 * MB,
     ie_workaround=True,
-) -> Iterator[Tuple[str, Union["FileUpload", str]]]:
+) -> Iterable[Tuple[str, Union["FileUpload", str]]]:
     """\
     Parse the contents of an HTTP POST request, which may be either
     application/x-www-form-urlencoded or multipart/form-data encoded.
@@ -597,13 +279,11 @@ def parse_post(
         raise MissingContentLength()
 
     try:
+        bytestream = io_iterator(fp, CHUNK_SIZE, maxlen=content_length)
         if ct == "application/x-www-form-urlencoded":
             if content_length > max_size:
                 raise TooBig("Content Length exceeds permitted size")
-            return parse_querystring(
-                SizeLimitedInput(fp, content_length).read().decode("ASCII"),
-                charset,
-            )
+            return parse_querystring(b"".join(bytestream).decode("ASCII"), charset)
         else:
             if content_length > max_multipart_size:
                 raise TooBig("Content Length exceeds permitted size")
@@ -614,7 +294,7 @@ def parse_post(
                     "No boundary given in multipart/form-data content-type"
                 )
             return parse_multipart(
-                SizeLimitedInput(fp, content_length),
+                bytestream,
                 boundary.encode("ASCII"),
                 charset,
                 max_size,
@@ -624,7 +304,7 @@ def parse_post(
         raise RequestParseError("Payload contains non ascii data")
 
 
-def get_body_bytes(environ, max_size=16 * KB):
+def get_body_bytes(environ, max_size=16 * KB) -> bytes:
     """
     Read a single message body from environ['wsgi.input'], returning a bytes
     object.
@@ -636,14 +316,14 @@ def get_body_bytes(environ, max_size=16 * KB):
 
     if content_length > max_size:
         raise TooBig("Content Length exceeds permitted size")
-    return SizeLimitedInput(environ["wsgi.input"], content_length).read()
+    return b"".join(io_iterator(environ["wsgi.input"], maxlen=content_length))
 
 
 def parse_body(environ, fp, default_charset=None, max_size=16 * KB):
     """
     Parse the message
         (payload as a byte string, content-type, encoding)
-        """
+    """
     ct, charset, params = get_content_type_info(
         environ.get("CONTENT_TYPE", "text/plain")
     )
@@ -656,7 +336,7 @@ class HTTPMessage(Message):
 
 
 def parse_multipart(
-    fp, boundary, default_charset, max_size, ie_workaround=True
+    stream: ByteIterator, boundary, default_charset, max_size, ie_workaround=True
 ) -> Iterator[Tuple[str, Union["FileUpload", str]]]:
     """
     Parse data encoded as ``multipart/form-data``. Generate tuples of::
@@ -666,7 +346,7 @@ def parse_multipart(
     Where ``data`` will be a string in the case of a regular input field, or a
     ``FileUpload`` instance if a file was uploaded.
 
-    :param fp: input stream from which to read data
+    :param stream: input stream from which to read data
     :param boundary: multipart boundary string, as specified by the
                      ``Content-Disposition`` header
     :param default_charset: character set to use for encoding, if not specified
@@ -678,31 +358,33 @@ def parse_multipart(
                           broken content-disposition header encoding.
     """
     boundary_size = len(boundary)
-
     if boundary_size > 72:
         raise RequestParseError(
-            "Malformed boundary string: "
-            "must be no more than 70 characters, not "
-            "counting the two leading hyphens (rfc 2046)"
+            "Malformed boundary string: must be no more than 70 characters, "
+            "not counting the two leading hyphens (rfc 2046)"
         )
 
     assert (
-        boundary_size + 2 < CHUNK_SIZE
-    ), "CHUNK_SIZE cannot be smaller than the boundary string"
+        boundary_size + 4 < CHUNK_SIZE
+    ), "CHUNK_SIZE cannot be smaller than the boundary string length + 4"
 
-    if fp.read(2) != b"--":
+    peek = next(stream)
+    while len(peek) < boundary_size + 4:
+        peek += next(stream)
+
+    if peek[0:2] != b"--":
         raise RequestParseError("Malformed POST data: expected two hypens")
 
-    if fp.read(boundary_size) != boundary:
+    if peek[2 : boundary_size + 2] != boundary:
         raise RequestParseError("Malformed POST data: expected boundary")
 
-    if fp.read(2) != b"\r\n":
+    if peek[boundary_size + 2 : boundary_size + 4] != b"\r\n":
         raise RequestParseError("Malformed POST data: expected CRLF")
 
-    fp = PutbackInput(fp)
+    stream = chain([peek[boundary_size + 4 :]], stream)
 
     while True:
-        headers, data = _read_multipart_field(fp, boundary)
+        headers, data, stream = _read_multipart_field(stream, boundary, max_size)
         try:
             _, params = parse_header(
                 headers["Content-Disposition"], ie_workaround=ie_workaround
@@ -713,74 +395,131 @@ def parse_multipart(
         try:
             name = params["name"]
         except KeyError:
-            raise RequestParseError(
-                "Missing name parameter in " "Content-Disposition header"
-            )
+            raise RequestParseError("Missing name in Content-Disposition header")
 
         is_file_upload = "Content-Type" in headers and "filename" in params
-
         if is_file_upload:
-            storage = data.getstorage()
-            storage.seek(0)
-            yield name, FileUpload(params["filename"], headers, storage)
-
+            data.seek(0)
+            yield name, FileUpload(params["filename"], headers, data)
         else:
             charset = parse_header(headers.get("Content-Type", ""))[1].get(
                 "charset", default_charset
             )
             if data.tell() > max_size:
+                data.close()
                 raise TooBig("Data block exceeds maximum permitted size")
             try:
                 data.seek(0)
                 yield name, data.read().decode(charset)
             except UnicodeDecodeError:
-                raise RequestParseError(
-                    "Invalid character data: can't decode "
-                    "as %r" % (charset,)
-                )
+                raise RequestParseError(f"Invalid {charset} character data")
 
-        chunk = fp.read(2)
-        if chunk == b"--":
-            if fp.peek(3) != b"\r\n":
-                raise RequestParseError(
-                    "Expected terminating CRLF " "at end of stream"
-                )
-            break
-
-        if chunk != b"\r\n":
-            raise RequestParseError("Expected CRLF after boundary")
+        peek = next(stream)
+        if peek[:2] == b"\r\n":
+            stream = chain([peek[2:]], stream)
+        elif peek == b"--\r\n":
+            if next(stream, None) is None:
+                break
+            else:
+                RequestParseError("Boundary not properly terminated")
+        else:
+            raise RequestParseError("Boundary not properly terminated")
 
 
 def _read_multipart_field(
-    fp, boundary, parser=BytesParser(_class=HTTPMessage)
-):
+    stream: ByteIterator, boundary: bytes, max_size: int
+) -> Tuple["HTTPMessage", SpooledTemporaryFile, ByteIterator]:
     """
     Read a single part from a multipart/form-data message and return a tuple of
-    ``(headers, data)``. Stream ``fp`` must be positioned at the start of the
-    header block for the field.
+    ``(headers, data, remainder)``.
+
+    Iterator ``iostream`` must be positioned at the start of the header block
+    for the field.
 
     Return a tuple of ('<headers>', '<data>')
 
     ``headers`` is an instance of ``email.message.Message``.
 
-    ``data`` is an instance of ``ExpandableOutput``.
-
-    Note that this currently cannot handle nested multipart sections.
+    ``data`` is an instance of ``tempfile.SpooledTemporaryFile``.
     """
-    data = ExpandableOutput()
-    headers = parser.parse(DelimitedInput(fp, b"\r\n\r\n"), headersonly=True)
-    fp = DelimitedInput(fp, b"\r\n--" + boundary)
+    output = SpooledTemporaryFile(max_size)
+    parser = BytesFeedParser(_factory=HTTPMessage)
+    parser._set_headersonly()  # type: ignore
 
-    # XXX: handle base64 encoding etc
-    for chunk in iter(lambda: fp.read(CHUNK_SIZE), b""):
-        data.write(chunk)
-    data.flush()
+    header_block, remainder, found = read_until(iter(stream), b"\r\n\r\n")
+
+    deque(map(parser.feed, header_block), maxlen=1)
+    if not found():
+        raise RequestParseError("Incomplete data (expected header)")
+    headers = parser.close()
+
+    sep = b"\r\n--" + boundary
+    data, remainder, found = read_until(remainder, sep)
+    for chunk in data:
+        output.write(chunk)
 
     # Fallen off the end of the input without having read a complete field?
-    if not fp.delimiter_found:
+    if not found():
+        output.close()
         raise RequestParseError("Incomplete data (expected boundary)")
+    output.flush()
 
-    return headers, data
+    return headers, output, remainder
+
+
+def read_until(
+    stream: Iterator[bytes], delimiter: bytes
+) -> Tuple[ByteIterator, ByteIterator, Callable[[], bool]]:
+    """
+    Return two iterators over byte stream `stream`` and a callable which
+    indicates whether the delimiter was found.
+
+    The first iterator yields all data up to ``delimiter``.
+    The second iterator generates all remaining data.
+
+    The first iterator must be exhausted before the second is iterated.
+    The callable must only be called after the first iterator has been
+    exhausted.
+    """
+    buf = b""
+    found = None
+
+    def _found():
+        if found is None:
+            raise AssertionError("The first iterator was not exhausted")
+        return found
+
+    def read_upto():
+        nonlocal buf, found
+        dlen = len(delimiter)
+        for chunk in chain(stream, [b""]):
+            buf += chunk
+            is_at_end = chunk == b""
+            if len(buf) > dlen + 4096 or is_at_end:
+                before, sep, after = buf.partition(delimiter)
+                if sep == b"":
+                    if is_at_end:
+                        found = False
+                        yield buf
+                        return
+                    pos = len(buf) - dlen
+                    if pos > 0:
+                        yield buf[:pos]
+                        buf = buf[pos:]
+                else:
+                    found = True
+                    if before:
+                        yield before
+                    buf = after
+                    return
+
+    def remainder():
+        nonlocal buf
+        if buf:
+            yield buf
+        yield from stream
+
+    return read_upto(), remainder(), _found
 
 
 class FileUpload(object):
@@ -875,10 +614,7 @@ def encode_multipart(data=None, files=None, charset="UTF-8", **kwargs):
     boundary = b"-------" + hexlify(os.urandom(16))
     alldata = chain(
         ((header_block(k), payload) for k, payload in data),
-        (
-            (file_header_block(k, fn, ct), payload)
-            for k, fn, ct, payload in files
-        ),
+        ((file_header_block(k, fn, ct), payload) for k, fn, ct, payload in files),
     )
 
     CRLF = b"\r\n"
