@@ -30,8 +30,6 @@ from fresco.typing import HeadersList
 from fresco.typing import WSGICallable
 
 __all__ = [
-    "environ_to_unicode",
-    "unicode_to_environ",
     "environ_to_str",
     "str_to_environ",
     "environ_to_bytes",
@@ -110,17 +108,6 @@ REQUEST_HEADER_NAMES = {
 }
 
 
-def with_docstring_from(src):
-    def with_docstring_from(tgt):
-        try:
-            tgt.__doc__ = src.__doc__
-        except AttributeError:
-            tgt.func_doc = src.func_doc
-        return tgt
-
-    return with_docstring_from
-
-
 def environ_to_str(s, enc="UTF-8"):
     """
     Decode a WSGI environ value to a string
@@ -140,16 +127,6 @@ def str_to_environ(s, enc="UTF-8"):
     return s.encode(enc, "surrogateescape").decode("iso-8859-1")
 
 
-def environ_to_unicode(s, enc="UTF-8"):
-    import warnings
-
-    warnings.warn(
-        "environ_to_unicode is deprecated, use environ_to_str instead",
-        DeprecationWarning,
-    )
-    return environ_to_str(s, enc)
-
-
 def unicode_to_environ(s, enc="UTF-8"):
     import warnings
 
@@ -187,7 +164,7 @@ def getenv(environ, key, default=None):
         return default
 
 
-def setenv(environ, key, value, enc="UTF-8"):
+def setenv(environ, key, value):
     """
     Set the named ``key`` to ``value`` in the WSGI environ dict ``environ``.
     Value must be a byte ``str`` and will be encoded according
@@ -211,8 +188,8 @@ class StartResponseWrapper(object):
         >>> def my_other_wsgi_app(environ, start_response):
         ...     responder = StartResponseWrapper(start_response)
         ...     result = my_wsgi_app(environ, responder)
-        ...     print "Got status", responder.status
-        ...     print "Got headers", responder.headers
+        ...     print("Got status", responder.status)
+        ...     print("Got headers", responder.headers)
         ...     responder.call_start_response()
         ...     return result
         ...
@@ -274,11 +251,11 @@ class ClosingIterator(object):
         >>> class filelikeobject(object):
         ...
         ...     def read(self):
-        ...         print "file read!"
+        ...         print("file read!")
         ...         return ''
         ...
         ...     def close(self):
-        ...         print "file closed!"
+        ...         print("file closed!")
         ...
         >>> def app(environ, start_response):
         ...     f = filelikeobject()
